var generateGameContent = generateGameContent || {};

generateGameContent.generateGrid = function( columns , rows ){

	/*
	*	Going to draw a basic grid of Columns * Rows.
	*
	*/

	// geometry is a blank slate object class whereby the user can add vertices and faces manually allowing a custom shape.
	var geometry = new THREE.Geometry();

	// since we want to visually display a grid for our playing field we'll use the LineBasicMaterial in combination
	// with the LineSegments Object below.

	for( var i=0, x= -( columns * 0.05 ), z= -( rows * 0.05 ) , col=0 , row=0 , ii = rows * columns; i<ii; i++ ){

		// we want the grid to be centered, so if we want 20 rows with 20 columns then the starting 
		// x position will be -1 and the starting z position will be -1;

		/*
		* NOTE:
		* we're also going to use col and row to keep track of our grid construction. May seem redundent, however;
		* since we are multiplying x and z by 0.05 we will sometimes get float numbers that will screw up
		* our grid with things like 1.399999999 or 1.40000000001 if we rely on them to know when a particular
		* grid or row is finished.
		*/

		// each line segment needs to be made of a starting and ending vertice.
		// since we'll want to highlight an individual square when placing towers we'll want
		// to draw each square visible independently.

		// lines going from:

		// back to front
		geometry.vertices.push( new THREE.Vector3( x , 0 , z+0.1 ) , new THREE.Vector3( x , 0 , z ) );

		// left to right
		geometry.vertices.push( new THREE.Vector3( x , 0 , z ) , new THREE.Vector3( x+0.1 , 0 , z ) );

		x += 0.1, col++;

		if( col > columns-1 ){

			// we've drawn a complete row of half boxes, now we need to add a final line to complete the last box's edge
			// back to forward.
			geometry.vertices.push( new THREE.Vector3( x , 0 , z+0.1 ) , new THREE.Vector3( x , 0 , z ) );
			
			// reset x to the far left and increment z;
			z += 0.1, x = -( columns * 0.05 ), col = 0, row++;

			if( row > rows-1 ){

				x = -( columns * 0.05 );
				for( var j=0; j<columns; j++ ){

					// we've drawn all boxes except for the last row's final edges.
					geometry.vertices.push( new THREE.Vector3( x , 0 , z ) , new THREE.Vector3( x+0.1 , 0 , z ) );
					x+= 0.1;
				}
			}
		}
	}

	// we'll set the basic color of the grid to a soft gray.  
	// Adding the vertex color parameter will allow us to adjust the color of each vertice manually,
	// this will come in handy later when we want to show we've selected an individual square and what not.
	var lineMaterial = new THREE.LineBasicMaterial( { vertexColors: THREE.VertexColors } );

	// add one color for each vectice in our geometry.
	for( var i=0, ii=geometry.vertices.length; i<ii; i+=2 ){
		geometry.colors[ i ] = new THREE.Color( 0.3 , 0.3 , 0.3 );
    	geometry.colors[ i + 1 ] = geometry.colors[ i ];
	}

	/* TRY RANDOM COLORS
	// random color generation for the grid
	for( var i=0, ii=geometry.vertices.length; i<ii; i+=2 ){
		geometry.colors[ i ] = new THREE.Color( Math.random(), Math.random(), Math.random() );
    	geometry.colors[ i + 1 ] = geometry.colors[ i ];
	}
	//*/

	var grid = new THREE.LineSegments( geometry , lineMaterial );

	// Alternatively, if we were to use the THREE.Line() object class lines would be drawn sequentially from vertice to vertice
	/* TRY EXAMPLE 
	grid = new THREE.Line( geometry, lineMaterial );
	//*/

	// we're going to add some custom values to the grid so that later on we can retrieve them
	grid.columns = columns;
	grid.rows = rows;

	return grid;
}

generateGameContent.generateGridTargetMesh = function( grid ){

	// since we already generated the vertices for grid, we'll use that vectorr list to copy
	// vectors to our target mesh.
	var v = 0; // used as a vertices counter
	var faceOne, faceTwo;
	var vectorList = grid.geometry.vertices;

	var geometry = new THREE.Geometry();
	var basicMaterial = new THREE.MeshLambertMaterial( { color: 0x0000ff , visible: false , side: THREE.DoubleSide });
	/* VIEW MESH
	basicMaterial.visible = true;
	//*/

	for( var i=0, ii=vectorList.length; i<ii; i++ ){
		geometry.vertices.push( vectorList[i].clone() );
	}

	// now we'll create 2 faces for each square, but we'll need to know how many rows and columns there are.
	for( var i=0, ii=grid.rows, f=0/* f is used as a local face counter */; i<ii; i++ ){
		for( var j=0, jj=grid.columns; j<jj; j++ ){

			// reference the vertices index position to draw the faces.
			faceOne = new THREE.Face3( v , v+3 , v+1 );
			faceOne.index = f;

			faceTwo = new THREE.Face3( v , v+4 , v+3 );
			faceTwo.index = f;

			geometry.faces.push( faceOne , faceTwo );
			f++;
			v+=4;
		}
		v+=2;

	}

	// in order for the targeting to work later on we'll need to do the following calculations
	geometry.computeFaceNormals();
	geometry.computeBoundingSphere();

	var gridTargetMesh = new THREE.Mesh( geometry , basicMaterial );

	return gridTargetMesh;
}

generateGameContent.createCells = function( grid ){

	var cells = [];

	// lineVertices: for each line drawing each side of each visual cell there are two vertices.  
	// 				 one for the start and end points.
	// surroundingCells: Indexed position of all surrounding cells in a clockwise fashion starting to the left.
	// 					 each number represents the index location within surroundingCells that contains the index 
	// 					 location of the cell within cell.list[].
	//
	//    | 1 | 2 | 3 |		----> example 		| 43 | 44 | 45 |
	//    | 0 | x | 4 |							| 60 | xx | 62 |
	// 	  | 7 | 6 | 5 |							| 79 | 78 | 77 |	----> will appear in the array as follows . . . 
	// 
	//	  . . [ 60 , 43 , 44 , 45 , 62 , 77 , 78 , 79 ]
	//
	// travelTo: is the passage from this cell to the indexed cell in surroundingCells allowed?
	// currentIteration: at the beginning of the pathfinding function this array is set to equal travelTo for each given cell
	// 					 then it is updated through each iteration for each cell and travelTo is never adjusted.

	// we're going to establish cell index/ID -1.  This will act as a stop for later logic.
	cells[ -1 ] = {
		lineVertices: [],
		surroundingCells: [ -1,-1,-1,-1,-1,-1,-1,-1 ],
		travelTo: [ false, false, false, false, false, false, false, false ],
		currentIteration: [ false, false, false, false, false, false, false, false ],
	}

	var vectorList = grid.geometry.vertices;
	var v = 0; // used as vertices counter;
	var c = 0; // used as cell counter;

	// we'll need to know how many rows and columns there are.

	// the first cell of each row will be a goal cell.  If the enemy make it to a goal square it will have escaped.

	// create first cell of first row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
		surroundingCells: [ -1,-1,-1,-1, 1 , grid.columns+1 , grid.columns , -1 ],
		travelTo: [ false, false, false, false, true, true, true, false ],
		currentIteration: [ false, false, false, false, true, true, true, false ],
		goal: true,
	}
	v+=4;
	c++;

	//generate first row minus first and last cell
	for( var j=1, jj=grid.columns-1; j<jj; j++ ){

		// create object for each new cell with the following:
		// - index position of vertices needed to draw line segements.
		// - index position of surrounding cells.

		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
			surroundingCells: [ c-1 , -1,-1,-1, c+1 , c + grid.columns + 1 , c + grid.columns , c + grid.columns - 1 ],
			travelTo: [ true, false, false, false, true, true, true, true ],
			currentIteration: [ true, false, false, false, true, true, true, true ],
			goal: false,
		}
		v+=4;
		c++;
	}

	// create last cell of first row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
		surroundingCells: [ c-1, -1,-1,-1,-1,-1 , c + grid.columns , c + grid.columns-1 ],
		travelTo: [ true, false, false, false, false, false, true, true ],
		currentIteration: [ true, false, false, false, false, false, true, true ],
		goal: false,
	}
	v+=6;
	c++;

	for( var i=1, ii=grid.rows-1; i<ii; i++ ){

		// create first cell of row
		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
			surroundingCells: [ -1,-1,
				c - grid.columns ,	
				c - ( grid.columns - 1 ),
				c + 1 , 
				c + ( grid.columns + 1 ) , 
				c + grid.columns , -1				
			],
			travelTo: [ false, false, true, true, true, true, true, false ],
			currentIteration: [ false, false, true, true, true, true, true, false ],
			goal: true,
		}
		v+=4;
		c++;

		for( var j=1, jj=grid.columns-1; j<jj; j++ ){

			// create object for each new cell with the following:
			// - index position of vertices needed to draw line segements.
			// - index position of surrounding cells.

			cells[c] = {
				lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
				surroundingCells: [ 
					c - 1 ,
					c - ( grid.columns + 1 ),
					c - grid.columns ,
					c - ( grid.columns - 1 ),
					c + 1 ,
					c + ( grid.columns + 1 ) , 
					c + grid.columns , 
					c + ( grid.columns - 1 )					
				],
				travelTo: [ true, true, true, true, true, true, true, true ],
				currentIteration: [ true, true, true, true, true, true, true, true ],
				goal: false,
			}
			v+=4;
			c++;
		}

		// create last cell of row
		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
			surroundingCells: [ 
				c - 1 ,
				c - ( grid.columns + 1 ),
				c - grid.columns , -1,-1,-1,
				c + grid.columns , 
				c + ( grid.columns - 1 ) , 							
			],
			travelTo: [ true, true, true, false, false, false, true, true ],
			currentIteration: [ true, true, true, false, false, false, true, true ],
			goal: false,
		}
		v+=6;
		c++;
	}

	//create first cell of last row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 ],
		surroundingCells: [ -1,-1, c-grid.columns , c - ( grid.columns-1 ) , c+1, -1,-1,-1 ],
		travelTo: [ false, false, true, true, true, false, false, false ],
		currentIteration: [ false, false, true, true, true, false, false, false ],
		goal: true,
	}
	v+=4;
	c++;

	//generate last row minus first and last cell
	for( var j=1, jj=grid.columns-1; j<jj; j++ ){

		// create object for each new cell with the following:
		// - index position of vertices needed to draw line segements.
		// - index position of surrounding cells.

		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 ],
			surroundingCells: [  
				c - 1 ,
				c - ( grid.columns + 1 ),
				c - grid.columns ,
				c - ( grid.columns - 1 ),
				c + 1 , -1,-1,-1,
			],
			travelTo: [ true, true, true, true, true, false, false, false ],
			currentIteration: [ true, true, true, true, true, false, false, false ],
			goal: false,
		}
		v+=4;
		c++;
	}

	// create last cell of last row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 ],
		surroundingCells: [ c -1 , c - ( grid.columns + 1 ) , c - grid.columns , -1,-1,-1,-1,-1 ],
		travelTo: [ true, true, true, false, false, false, false, false ],
		currentIteration: [ true, true, true, false, false, false, false, false ],
		goal: false,
	}
	
	// add remaining vertices to last row
	c -= ( grid.columns-1 )
	v =  grid.geometry.vertices.length - ( grid.columns*2 );
	for( var i=0, ii=grid.columns; i<ii; i++ ){
		cells[c].lineVertices.push( v , v+1 );
		v+=2;
		c++;
	}

	return { list: cells, previousSelection: [], info: { side:-1, cellID:-1 } };
}

//////////////////////////////////////////////////////////////

generateGameContent.createGameParameters = function(){

	// these paraments will be used throughout

	var gameParaments = {

		gridRows: 7,	
		gridColumns: 13, 
		totalWalls: 40,

	}

	return gameParaments;
}

generateGameContent.createUserSelectionTaskManager = function(){

	// within the task manager we'll keep track of various 
	// user selected tasks such as placing towers, upgrading,
	// menu selections and so on.

	// Since we're going to add a graphical user interface at 
	// a later time we're going to allow for functionality 
	// through hot keys.  

	var userTaskManager = {

		// if a user presses the same button twice then it will cancel the selection.

		createWalls: false, // hotkey w , 87
		createTowers: false, // hotkey t , 84

	}

	return userTaskManager;
}

generateGameContent.generateWallMesh = function(){

	var geometry = new THREE.Geometry();

	var wallMaterial = new THREE.MeshLambertMaterial( { color: 0x444477 /*, side: THREE.DoubleSide*/ } );

	for( var i=0, ii=gameParameters.totalWalls*4; i<ii; i+=4 ){

		// add four vertices for each corner of a wall
		geometry.vertices.push( 	new THREE.Vector3( 0 , 0 , 0 ) ,
								new THREE.Vector3( 0 , 0 , 0 ) , 
								new THREE.Vector3( 0 , 0 , 0 ) ,
								new THREE.Vector3( 0 , 0 , 0 ) );

		// add two faces for the wall, since the material is double sided we only need two
		geometry.faces.push( new THREE.Face3( i+1 , i , i+2 ) , new THREE.Face3( i+1 , i+2 , i+3 ) );

	}

	var walls = new THREE.Mesh( geometry , wallMaterial );

	// we're going to add a custom array to walls.
	// it will serve as a look up array to see if a wall already exists at a given vertice.
	walls.lookUpArray = [];

	// a bufferIndex to let us know what current index position to use for newly placed walls
	walls.bufferIndex = { vertices: 0 , maxVertices: ( gameParameters.totalWalls*4 ) };

	// a link array telling us which line segement is associated with which buffer index position
	// will come in handy when removing walls.
	walls.bufferLink = [];

	// an available index array incase the user wants to remove a wall segment.
	// will come in handy when removing walls.
	walls.availableIndexes = [];

	return walls;	
}

//////////////////////////////////////////////////////////////